/* ARCADEmini simulator (unix, console)
 * version 1.0.2 date 2005-10-24
 * part of the ARCADEmini project - http[s]://arcademini.schuermans.info/
 * Copyright (C) 2003-2005 sphaera <sphaera@geekness.de>
 *                     and 1stein <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "am_dev.h"
#include "display.h"
#include "input.h"

int end = 0;

//signal handler to end the program
void end_signal( int sig_no )
{
  sig_no += 0; //keep compiler-happy
  end = 1;
}

//main program
int main( int param_cnt, char * * params )
{
  char * input_file, * log_file;
  int param_err, param_no;
  FILE * p_file, * p_log_file;
  st_am_state am_state;
  unsigned short msec;

  //issue short info
  puts( "ARCADEmini simulator (unix, console)\n"
        "version 1.0.2 date 2005-10-24\n"
        "part of the ARCADEmini project - http[s]://arcademini.schuermans.info/\n"
        "Copyright (C) 2003-2005 sphaera <sphaera@geekness.de>\n"
        "                     and 1stein <1stein@schuermans.info>\n"
        "Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html\n"
      );

  //read parameters
  param_err = 0;
  input_file = NULL;
  log_file = NULL;
  for( param_no = 1; param_no < param_cnt; param_no++ )
  {
    //logfile filename
    if( strcmp( params[param_no], "-l" ) == 0 || strcmp( params[param_no], "--log" ) == 0 )
    {
      param_no++;
      if( log_file != NULL || param_no >= param_cnt ) //already specified or no filename following
        param_err = 1;
      else
        log_file = params[param_no];
    }
    //unknown option
    else if( params[param_no][0] == '-' )
      param_err = 1;
    //input filename
    else
    {
      if( input_file != NULL ) //already specified
        param_err = 1;
      else
        input_file = params[param_no];
    }
  } //for( param_no ...

  //check parameters
  if( input_file == NULL )
    param_err = 1;

  //parameter error
  if( param_err )
  {
    //print syntax explanation
    puts( "syntax:     am_sim [options] <input-file>\n"
          "\n"
          "options:    -l / --log <logfile>\n"
          "\n"
          "example:    am_sim -l test.log test.ambin\n"
          "\n"
        );
    //exit with error
    return -1;
  }

  //open input file
  p_file = fopen( input_file, "rb" );
  if( p_file == NULL )
  {
    printf( "could not open input-file \"%s\"\n", input_file );
    return -1;
  }

  //open logfile
  if( log_file == NULL )
    p_log_file = NULL;
  else
  {
    p_log_file = fopen( log_file, "wt" );
    if( p_log_file == NULL )
    {
      printf( "could not open logfile \"%s\"\n", log_file );
      return -1;
    }
  }

  //initialize state
  if( am_dev_init_state( &am_state, p_file, p_log_file ) != 0 )
  {
    printf( "error in ambin-file \"%s\"\n", input_file );
    return -1;
  }

  //install signal handler to end program
  signal( SIGHUP, end_signal );
  signal( SIGINT, end_signal );
  signal( SIGTERM, end_signal );

  //initialize input
  if( input_init( ) != 0 )
  {
    fclose( p_file );
    if( p_log_file != NULL )
      fclose( p_log_file );
    return -1;
  }

  //initialize display
  if( display_init( ) != 0 )
  {
    input_exit( );
    fclose( p_file );
    if( p_log_file != NULL )
      fclose( p_log_file );
    return -1;
  }

  //main loop
  while( !end )
  {
    //execute a command
    msec = am_dev_exec( &am_state );
    //display new picture if it changed
    if( am_state.picture_changed )
    {
      display_picture( &am_state );
      am_state.picture_changed = 0;
    }
    //wait for delay to end
    if( msec > 0 )
    {
      if( msec > 50 )
        usleep( 50000 );
      else
        usleep( (unsigned long)msec * 1000 );
    }
    //poll display - check if end is requested
    if( ! end )
      if( display_poll( &am_state ) )
        end = 1;
    //poll input - check if end is requested
    if( ! end )
      if( input_poll( &am_state ) )
        end = 1;
  } //while( !end )

  //cleanup display and input
  display_exit( );
  input_exit( );

  //close input-file
  fclose( p_file );

  //close logfile
  if( p_log_file != NULL )
    fclose( p_log_file );

  //end
  display_final( );
  return 0;
}
